import os
import sys
import optparse
import time
import json
import numpy as np
from sumo_util import *

# we need to import some python modules from the $SUMO_HOME/tools directory
if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")


from sumolib import checkBinary  # Checks for the binary in environ vars
import traci

def save_files(phase_dict,current_duration,current_index):
    save_data('saved_trafficlight.json', phase_dict)
    save_data('saved_currentDuration.json', current_duration)
    save_data('saved_currentIndex.json', current_index)
    traci.simulation.saveState('saved_simulation')
    traci.close()

def load_files(phase_dict,current_duration,current_index):
    phase_dict = load_data('saved_trafficlight.json')
    current_index = load_data('saved_currentIndex.json')
    current_duration = load_data('saved_currentDuration.json')
    traci.start([checkBinary('sumo-gui'), "-c", "config.sumocfg", "--load-state", 'saved_simulation']) 

def parse_phase():
    phase_dict = {}
    phase_info = []
    junction_IDs = traci.trafficlight.getIDList()
    for id in range(len(junction_IDs)):
        phase_dict[junction_IDs[id]] = []
        phase_info = []
        complete_phase = str(traci.trafficlight.getCompleteRedYellowGreenDefinition(junction_IDs[id])[0])
        # print(complete_phase)
        for x in complete_phase.split('phase')[1].split('state')[1:]:
            for y in x.split(',')[0:2]:
                phase_info.append(y.split('=')[1].strip("'"))
        for x in range(0,len(phase_info),2):
            phase_dict[junction_IDs[id]].append(phase_info[x])
    return phase_dict


def run():
    step = 0
    phase_dict = parse_phase()
    current_index = {}
    current_duration = {}
    junction_IDs = list(phase_dict.keys())
    for ID in junction_IDs:
        current_index[ID] = 0
        current_duration[ID] = 10

    vehID = "addtionVehical."+str(0)
    traci.vehicle.add(vehID, "route_90", "car")
    vehID = "addtionVehical."+str(1)
    traci.vehicle.add(vehID, "route_90", "car")
    
    while traci.simulation.getMinExpectedNumber() > 0:
        for junc in phase_dict:
            traci.trafficlight.setPhase(junc, current_index[junc])
        reduction_time = sorted(current_duration.items(), key=lambda x: x[1])[0][1]
        i = 0
        while i < reduction_time:
            traci.simulationStep()
            i += traci.simulation.getDeltaT()
            step += 1

            if step == 195: 
                save_files(phase_dict,current_duration,current_index)
                load_files(phase_dict,current_duration,current_index)
                for junc in phase_dict:
                    traci.trafficlight.setPhase(junc, current_index[junc])
                    traci.trafficlight.setPhaseDuration(junc, current_duration[junc])
        
        for juncID in junction_IDs:
            current_duration[juncID] -= reduction_time
        
        for juncID in junction_IDs:
            if current_duration[juncID] == 0:
                current_index[juncID] = (current_index[juncID] + 1) % len(phase_dict[juncID])
                if 'y' in phase_dict[juncID][current_index[juncID]]:
                    current_duration[juncID] = 5
                else:
                    current_duration[juncID] = 10
    traci.close()
    print('STEP',step)
    sys.stdout.flush()

# main entry point
if __name__ == "__main__":
    traci.start([checkBinary('sumo-gui'), "-c", "config.sumocfg",
                              "--step-length", "0.5"]) #"--remote-port", "65000"

    run()
    
